@@ -5,7 +5,7 @@ package OpenAPI::Modern;
 # ABSTRACT: Validate HTTP requests and responses against an OpenAPI document
 # KEYWORDS: validation evaluation JSON Schema OpenAPI Swagger HTTP request response
 
-our $VERSION = '0.020';
+our $VERSION = '0.021';
 
 use 5.020;  # for fc, unicode_strings features
 use Moo;
@@ -23,7 +23,7 @@ use Scalar::Util 'looks_like_number';
 use Feature::Compat::Try;
 use Encode 2.89;
 use URI::Escape ();
-use JSON::Schema::Modern 0.531;
+use JSON::Schema::Modern 0.543;
 use JSON::Schema::Modern::Utilities 0.531 qw(jsonp unjsonp canonical_uri E abort is_equal);
 use JSON::Schema::Modern::Document::OpenAPI;
 use MooX::HandlesVia;
@@ -83,6 +83,7 @@ sub validate_request ($self, $request, $options = {}) {
     traversed_schema_path => '',    # the accumulated traversal path as of the start, or last $id, or up to the last traversed $ref
     schema_path => '',              # the rest of the path, since the last $id or the last traversed $ref
     errors => $options->{errors} //= [],
+    effective_base_uri => Mojo::URL->new->host(scalar _header($request, 'Host'))->scheme('https'),
   };
 
   try {
@@ -182,6 +183,8 @@ sub validate_response ($self, $response, $options = {}) {
     die pop $options->{errors}->@*
       if not $self->find_path($response->$_call_if_can('request') // $options->{request}, $options);
 
+    $state->{effective_base_uri} = Mojo::URL->new->host(scalar _header($options->{request}, 'Host'))->scheme('https')
+      if $options->{request};
     my ($path_template, $path_captures) = $options->@{qw(path_template path_captures)};
     my $method = lc $options->{method};
     my $operation = $self->openapi_document->schema->{paths}{$path_template}{$method};
@@ -246,6 +249,7 @@ sub find_path ($self, $request, $options) {
     traversed_schema_path => '',    # the accumulated traversal path as of the start, or last $id, or up to the last traversed $ref
     schema_path => '',              # the rest of the path, since the last $id or the last traversed $ref
     errors => $options->{errors} //= [],
+    $request ? ( effective_base_uri => Mojo::URL->new->host(scalar _header($request, 'Host'))->scheme('https') ) : (),
   };
 
   # method from options
@@ -258,6 +262,8 @@ sub find_path ($self, $request, $options) {
     $method = lc $request->method;
   }
 
+  $options->{request} = $request if $request;
+
   # path_template and method from operation_id from options
   if (exists $options->{operation_id}) {
     my $operation_path = $self->openapi_document->get_operationId($options->{operation_id});
@@ -562,6 +568,7 @@ sub _evaluate_subschema ($self, $data, $schema, $state) {
     {
       data_path => $state->{data_path},
       traversed_schema_path => $state->{traversed_schema_path}.$state->{schema_path},
+      effective_base_uri => $state->{effective_base_uri},
     },
   );
 
@@ -586,6 +593,7 @@ sub _query_pairs ($uri) {
 
 # note: this assumes that the header values were already normalized on creation,
 # as sanitizing on read is bypassed
+# beware: the lwp version is list/scalar-context-sensitive
 sub _header ($message, $header_name) {
     $message->isa('HTTP::Message') ? $message->headers->header($header_name)
   : $message->isa('Mojo::Message') ? $message->content->headers->header($header_name) // ()
@@ -636,12 +644,12 @@ OpenAPI::Modern - Validate HTTP requests and responses against an OpenAPI docume
 
 =head1 VERSION
 
-version 0.020
+version 0.021
 
 =head1 SYNOPSIS
 
   my $openapi = OpenAPI::Modern->new(
-    openapi_uri => 'openapi.yaml',
+    openapi_uri => '/api',
     openapi_schema => YAML::PP->new(boolean => 'JSON::PP')->load_string(<<'YAML'));
   openapi: 3.1.0
   info:
@@ -691,11 +699,9 @@ version 0.020
                       const: ok
   YAML
 
-  use HTTP::Request::Common;
-  use Mojo::Message::Response;
   say 'request:';
-  my $request = POST 'http://example.com/foo/bar',
-    'My-Request-Header' => '123', 'Content-Type' => 'application/json',
+  my $request = POST '/foo/bar',
+    'My-Request-Header' => '123', 'Content-Type' => 'application/json', Host => 'example.com',
     Content => '{"hello": 123}';
   my $results = $openapi->validate_request($request);
   say $results;
@@ -707,6 +713,7 @@ version 0.020
   $response->headers->header('Content-Type', 'application/json');
   $response->headers->header('My-Response-Header', '123');
   $response->body('{"status": "ok"}');
+  $results = $openapi->validate_response($response, { request => $request });
   say $results;
   say ''; # newline
   say JSON::MaybeXS->new(convert_blessed => 1, canonical => 1, pretty => 1, indent_length => 2)->encode($results);
@@ -720,13 +727,13 @@ prints:
   {
     "errors" : [
       {
-        "absoluteKeywordLocation" : "openapi.yaml#/paths/~1foo~1%7Bfoo_id%7D/post/requestBody/content/application~1json/schema/properties/hello/type",
+        "absoluteKeywordLocation" : "https://example.com/api#/paths/~1foo~1%7Bfoo_id%7D/post/requestBody/content/application~1json/schema/properties/hello/type",
         "error" : "got integer, not string",
         "instanceLocation" : "/request/body/hello",
         "keywordLocation" : "/paths/~1foo~1{foo_id}/post/requestBody/content/application~1json/schema/properties/hello/type"
       },
       {
-        "absoluteKeywordLocation" : "openapi.yaml#/paths/~1foo~1%7Bfoo_id%7D/post/requestBody/content/application~1json/schema/properties",
+        "absoluteKeywordLocation" : "https://example.com/api#/paths/~1foo~1%7Bfoo_id%7D/post/requestBody/content/application~1json/schema/properties",
         "error" : "not all properties are valid",
         "instanceLocation" : "/request/body",
         "keywordLocation" : "/paths/~1foo~1{foo_id}/post/requestBody/content/application~1json/schema/properties"
@@ -761,6 +768,9 @@ than features that seem to work but actually cut corners for simplicity.
 The URI that identifies the OpenAPI document.
 Ignored if L</openapi_document> is provided.
 
+If it is not absolute, it is resolved at runtime against the request's C<Host> header (when available)
+and the https scheme is assumed.
+
 =head2 openapi_schema
 
 The data structure describing the OpenAPI v3.1 document (as specified at
